#include <kos.h>
#include "dc.h"
#include "gui.h"

uint16 *guiFontTexture;

/* Sets up a texture for the font (yeah, I took a VERY good look
 * at ghettoplay)
 */

void gui_setup(){
  uint16 *vbuf = (uint16*)pvr_mem_malloc(256*256*2);
  int x, y;

  guiFontTexture = vbuf;

  memset(vbuf, 0, 256*256*2);
  for(y=0; y<8; y++) {
    for(x=0; x<16; x++) {
      bfont_draw(vbuf, 256, 0, y*16+x);
      vbuf+=16;
    }
    vbuf+=23*256;
  } 

}

void gui_shutdown(){
  pvr_mem_free(guiFontTexture);
}

void gui_begin(){
  dc_beginFrame();
  dc_blit();
  pvr_list_begin(PVR_LIST_TR_POLY);
}

void gui_end(){
  pvr_list_finish();
  dc_endFrame();
}

void gui_drawBox( uint32 c1, uint32 c2, float x1, float y1, 
		   float x2, float y2, float z ) {
  pvr_poly_hdr_t hdr;
  pvr_poly_cxt_t cxt;
  pvr_vertex_t vert;
  uint32 halfcol=0;
  int i;
  for(i=0;i<32;i+=8) {
    halfcol |= (((((c1>>i) & 0xff) + ((c2>>i) & 0xff))/2) << i);
  }

  pvr_poly_cxt_col(&cxt, PVR_LIST_TR_POLY);
  pvr_poly_compile( &hdr, &cxt );
  pvr_prim(&hdr, sizeof(hdr));

  vert.flags = PVR_CMD_VERTEX;
  vert.x = x1; vert.y = y2; vert.z = z;
  vert.argb = halfcol;
  vert.oargb = 0x00000000;
  pvr_prim(&vert, sizeof(vert));

  vert.y = y1;
  vert.argb = c1;
  pvr_prim(&vert, sizeof(vert));

  vert.x = x2; vert.y = y2;
  vert.argb = c2;
  pvr_prim(&vert, sizeof(vert));

  vert.flags = PVR_CMD_VERTEX_EOL;
  vert.y = y1;
  vert.argb = halfcol;
  pvr_prim(&vert, sizeof(vert));

}

void drawChar(uint32 color, float x, float y, float z, int c) {
  pvr_vertex_t vert;
  int ix = (c & 0xf) * 16;
  int iy = (c >> 4) * 24;
  float u1 = ix * 1.0f/256.0f;
  float v1 = iy * 1.0f/256.0f;
  float u2 = (ix + 12) * 1.0f / 256.0f;
  float v2 = (iy + 24) * 1.0f / 256.0f;

  vert.flags = PVR_CMD_VERTEX;
  vert.x = x;
  vert.y = y + 24.0f;
  vert.z = z;
  vert.u = u1;
  vert.v = v2;
  vert.oargb = 0x00000000;
  vert.argb = color;
  pvr_prim(&vert, sizeof(vert));

  vert.y = y;
  vert.v = v1;
  pvr_prim(&vert, sizeof(vert));

  vert.x = x + 12.0f;
  vert.y = y + 24.0f;
  vert.u = u2;
  vert.v = v2;
  pvr_prim(&vert, sizeof(vert));

  vert.flags = PVR_CMD_VERTEX_EOL;
  vert.y = y;
  vert.v = v1;
  pvr_prim(&vert, sizeof(vert));
}

static char strbuf[1024];
int gui_printf(uint32 col, float x, float y, float z, const char *fmt, ...) {
  pvr_poly_cxt_t cxt;
  pvr_poly_hdr_t hdr;
  int ret;
  va_list args;
  char *s=strbuf;

  va_start(args, fmt);
  ret = vsprintf(strbuf, fmt, args);
  va_end(args);

  pvr_poly_cxt_txr(&cxt, PVR_LIST_TR_POLY, 
		   PVR_TXRFMT_ARGB4444 | PVR_TXRFMT_NONTWIDDLED, 256, 256,
		   guiFontTexture, PVR_FILTER_NONE);

  pvr_poly_compile(&hdr, &cxt);
  
  pvr_prim(&hdr, sizeof(hdr));
  while(*s) {
    drawChar( col, x+=12.0f, y, z, *s++ );
  }
  return ret;
}

