///////////////////////////////////////////////////////////////////////////////
// Wonderswan emulator
////////////////////////////////////////////////////////////////////////////////
//
// 13.04.2002: Fixed a small bug causing crashes
//
//
//
//
//////////////////////////////////////////////////////////////////////////////

//#include <direct.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <malloc.h>
#include <time.h>

#include "log.h"
#include "rom.h"
#include "nec/nec.h"
#include "memory.h"
#include "gpu.h"
#include "io.h"
#include "ws.h"
#include "dc.h"
#include "menu.h"


////////////////////////////////////////////////////////////////////////////////
//
////////////////////////////////////////////////////////////////////////////////
//
//
//
//
//
//
//
////////////////////////////////////////////////////////////////////////////////

int			ws_colourScheme=COLOUR_SCHEME_DEFAULT;
int			ws_system=WS_SYSTEM_AUTODETECT;

char         		ws_rom_path[4096];
int ws_restart=0;
int ws_terminate=0;
int ws_reset_game=0;


static volatile int frameCount;

static void timer_frameCounter() {
  ++frameCount;
}

static void ws_emulation_mainloop() {
  
  int i=0, nCount;

  int16 *backbuffer;
  dc_setDisplay(2.0, ws_rotated());
  backbuffer = dc_getFrameBuffer();
  
  ws_reset_game = 0;

  dc_setTimerHandler(timer_frameCounter, 100);

  while (!(ws_restart || ws_reset_game || ws_terminate)) {

    //HACK
    //    frameCount=5;

    dc_getControls();
      
    if(dc_open_menu) {
      menu_open();
      frameCount = 0;
    }
    
    for (i=0;i<frameCount-1;i++) 
      while (!ws_executeLine(backbuffer,0));
    
    while (!ws_executeLine(backbuffer,1));

    frameCount = 0;
    
    dc_beginFrame();
    dc_blit();
    dc_dummyTransList();
    dc_endFrame();
  }

  dc_stopTimer();
}

/* Program entry */

int main () {
	
  fprintf(log_get(),"Wonderswan emulator v0.70 (last build on %s %s)\n",__DATE__,__TIME__);

  dc_init();
  memset(ws_rom_path, 0, 4096);

  while (!ws_terminate)
    {
     
      if (strlen(ws_rom_path)==0){
	  menu_open();
      } else {
	ws_restart = 0;
	if (ws_init(ws_rom_path)) {
	    
	    ws_set_system(ws_system);
	    ws_set_colour_scheme(ws_colourScheme);
	    while(!(ws_restart || ws_terminate)) {
	      ws_reset();
	      printf("Entering main loop...\n");
	      ws_emulation_mainloop();
	    }
	} else {
	  *ws_rom_path=0;
	}
	ws_done();
      }
    }
  
  log_done();
  dc_shutdown();
  return(0);
}
